@@ -1,13 +1,13 @@
 """Tests for compare.py."""
 
-from pytype import abstract
-from pytype import abstract_utils
 from pytype import compare
 from pytype import config
 from pytype import context
 from pytype import errors
-from pytype import function
 from pytype import load_pytd
+from pytype.abstract import abstract
+from pytype.abstract import abstract_utils
+from pytype.abstract import function
 from pytype.pytd import pytd
 from pytype.pytd import slots
 from pytype.tests import test_base
