@@ -174,7 +174,7 @@ class Converter(utils.ContextWeakrefMixin):
 
   def name_to_value(self, name, subst=None, ast=None):
     if ast is None:
-      pytd_cls = self.ctx.vm.lookup_builtin(name)
+      pytd_cls = self.ctx.loader.lookup_builtin(name)
     else:
       pytd_cls = ast.Lookup(name)
     subst = subst or datatypes.AliasingDict()
@@ -514,20 +514,25 @@ class Converter(utils.ContextWeakrefMixin):
   def _load_late_type(self, late_type):
     """Resolve a late type, possibly by loading a module."""
     if late_type.name not in self._resolved_late_types:
-      ast, attr_name = self._load_late_type_module(late_type)
-      if ast is None:
-        log.error("During dependency resolution, couldn't resolve late type %r",
-                  late_type.name)
-        t = pytd.AnythingType()
+      ast = self.ctx.loader.import_name(late_type.name)
+      if ast:
+        t = pytd.Module(name=late_type.name, module_name=late_type.name)
       else:
-        try:
-          cls = pytd_utils.LookupItemRecursive(ast, attr_name)
-        except KeyError:
-          if "__getattr__" not in ast:
-            log.warning("Couldn't resolve %s", late_type.name)
+        ast, attr_name = self._load_late_type_module(late_type)
+        if ast is None:
+          log.error(
+              "During dependency resolution, couldn't resolve late type %r",
+              late_type.name)
           t = pytd.AnythingType()
         else:
-          t = pytd.ToType(cls, allow_functions=True)
+          try:
+            cls = pytd.LookupItemRecursive(ast, attr_name)
+          except KeyError:
+            if "__getattr__" not in ast:
+              log.warning("Couldn't resolve %s", late_type.name)
+            t = pytd.AnythingType()
+          else:
+            t = pytd.ToType(cls, allow_functions=True)
       self._resolved_late_types[late_type.name] = t
     return self._resolved_late_types[late_type.name]
 
@@ -536,13 +541,16 @@ class Converter(utils.ContextWeakrefMixin):
       raise abstract_utils.ModuleLoadError()
     data = (ast.constants + ast.type_params + ast.classes +
             ast.functions + ast.aliases)
-    members = {val.name.rsplit(".")[-1]: val for val in data}
+    members = {}
+    for val in data:
+      name = utils.strip_prefix(val.name, f"{ast.name}.")
+      members[name] = val
     return abstract.Module(self.ctx, ast.name, members, ast)
 
   def _get_literal_value(self, pyval):
-    if pyval == self.ctx.vm.lookup_builtin("builtins.True"):
+    if pyval == self.ctx.loader.lookup_builtin("builtins.True"):
       return True
-    elif pyval == self.ctx.vm.lookup_builtin("builtins.False"):
+    elif pyval == self.ctx.loader.lookup_builtin("builtins.False"):
       return False
     elif isinstance(pyval, str):
       prefix, value = parser_constants.STRING_RE.match(pyval).groups()[:2]
@@ -648,7 +656,8 @@ class Converter(utils.ContextWeakrefMixin):
           function.PyTDSignature(pyval.name, sig, self.ctx)
           for sig in pyval.signatures
       ]
-      type_new = self.ctx.vm.lookup_builtin("builtins.type").Lookup("__new__")
+      type_new = self.ctx.loader.lookup_builtin("builtins.type").Lookup(
+          "__new__")
       if pyval is type_new:
         f_cls = special_builtins.TypeNew
       else:
@@ -663,7 +672,7 @@ class Converter(utils.ContextWeakrefMixin):
         # If pyval is a reference to a class in builtins or typing, we can fill
         # in the class ourselves. lookup_builtin raises a KeyError if the name
         # is not found.
-        cls = self.ctx.vm.lookup_builtin(pyval.name)
+        cls = self.ctx.loader.lookup_builtin(pyval.name)
         assert isinstance(cls, pytd.Class)
       return self.constant_to_value(cls, subst, self.ctx.root_node)
     elif isinstance(pyval, pytd.NothingType):
@@ -843,6 +852,9 @@ class Converter(utils.ContextWeakrefMixin):
             ret = attr_overlay.AttribInstance.from_metadata(
                 self.ctx, self.ctx.root_node, typ, md)
             return ret
+          elif md["tag"] == "attr.s":
+            ret = attr_overlay.Attrs.from_metadata(self.ctx, md)
+            return ret
         except (IndexError, ValueError, TypeError, KeyError):
           details = "Wrong format for pytype_metadata."
           self.ctx.errorlog.invalid_annotation(self.ctx.vm.frames,
