@@ -347,7 +347,7 @@ class ErrorLogBase:
   def copy_from(self, errors, stack):
     for e in errors:
       with _CURRENT_ERROR_NAME.bind(e.name):
-        self.error(stack, e.message, e.details, e.keyword, e.bad_call,
+        self.error(stack, e._message, e.details, e.keyword, e.bad_call,  # pylint: disable=protected-access
                    e.keyword_context)
 
   def is_valid_error_name(self, name):
@@ -923,14 +923,16 @@ class ErrorLog(ErrorLogBase):
     self.error(stack, message, details)
 
   @_error_name("bad-concrete-type")
-  def bad_concrete_type(self, stack, node, formal, actual, bad):
+  def bad_concrete_type(self, stack, node, formal, actual, bad, details=None):
     expected, actual, _, protocol_details, nis_details = (
         self._print_as_return_types(node, formal, actual, bad))
-    details = ["       Expected: ", expected, "\n",
-               "Actually passed: ", actual]
-    details.extend(protocol_details + nis_details)
+    full_details = ["       Expected: ", expected, "\n",
+                    "Actually passed: ", actual]
+    if details:
+      full_details.append("\n" + details)
+    full_details.extend(protocol_details + nis_details)
     self.error(
-        stack, "Invalid instantiation of generic class", "".join(details))
+        stack, "Invalid instantiation of generic class", "".join(full_details))
 
   def _show_variable(self, var):
     """Show variable as 'name: typ' or 'pyval: typ' if available."""
@@ -959,7 +961,7 @@ class ErrorLog(ErrorLogBase):
     args = " and ".join(str(operand) for operand in operands)
     if operator in slots.COMPARES:
       symbol = operator
-      details = f"Primitive types {args} are not comparable."
+      details = f"Types {args} are not comparable."
       self.error(stack, f"unsupported operand types for {symbol}",
                  details=details)
     else:
