@@ -80,9 +80,11 @@ class IOTest(unittest.TestCase):
       with self._tmpfile(
           "{mod} {path}".format(mod=pyi_name, path=pyi.name)) as imports_map:
         src = "import {mod}; y = {mod}.x".format(mod=pyi_name)
-        options = config.Options.create(imports_map=imports_map.name)
+        options = config.Options.create(imports_map=imports_map.name,
+                                        gen_stub_imports=True)
         _, pyi_string, _ = io.generate_pyi(src, options)
-    self.assertEqual(pyi_string, "{mod}: module\ny: int\n".format(mod=pyi_name))
+    self.assertEqual(pyi_string,
+                     "import {mod}\n\ny: int\n".format(mod=pyi_name))
 
   def test_check_or_generate_pyi__check(self):
     with self._tmpfile("") as f:
