@@ -417,7 +417,7 @@ class ImportPathsTest(_LoaderTest):
       """)
       loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
       foo = loader.import_name("foo")
-      self.assertEqual(pytd_utils.Print(foo), "import foo.bar as foo.baz")
+      self.assertEqual(pytd_utils.Print(foo), "from foo import bar as foo.baz")
 
   def test_typing_reexport(self):
     with file_utils.Tempdir() as d:
@@ -478,6 +478,18 @@ class ImportPathsTest(_LoaderTest):
       loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
       loader.import_name("foo.bar")
 
+  def test_module_alias(self):
+    ast = self._import(foo="""
+      import subprocess as _subprocess
+      x: _subprocess.Popen
+    """)
+    expected = textwrap.dedent("""
+      import subprocess as foo._subprocess
+
+      foo.x: _subprocess.Popen
+    """).strip()
+    self.assertMultiLineEqual(pytd_utils.Print(ast), expected)
+
 
 class ImportTypeMacroTest(_LoaderTest):
 
@@ -742,5 +754,82 @@ class MethodAliasTest(_LoaderTest):
                      "def b.f() -> int: ...")
 
 
+class RecursiveAliasTest(_LoaderTest):
+
+  def test_basic(self):
+    ast = self._import(a="""
+      from typing import List
+      X = List[X]
+    """)
+    actual_x = ast.Lookup("a.X")
+    expected_x = pytd.Alias(
+        name="a.X",
+        type=pytd.GenericType(
+            base_type=pytd.ClassType("builtins.list"),
+            parameters=(pytd.LateType("X"),)))
+    self.assertEqual(actual_x, expected_x)
+
+  def test_mutual_recursion(self):
+    ast = self._import(a="""
+      from typing import List
+      X = List[Y]
+      Y = List[X]
+    """)
+
+    actual_x = ast.Lookup("a.X")
+    expected_x = pytd.Alias(
+        name="a.X",
+        type=pytd.GenericType(
+            base_type=pytd.ClassType("builtins.list"),
+            parameters=(pytd.LateType("Y"),)))
+    self.assertEqual(actual_x, expected_x)
+
+    actual_y = ast.Lookup("a.Y")
+    expected_y = pytd.Alias(
+        name="a.Y",
+        type=pytd.GenericType(
+            base_type=pytd.ClassType("builtins.list"),
+            parameters=(
+                pytd.GenericType(
+                    base_type=pytd.ClassType("builtins.list"),
+                    parameters=(pytd.LateType("Y"),)),)))
+    self.assertEqual(actual_y, expected_y)
+
+  def test_very_mutual_recursion(self):
+    ast = self._import(a="""
+      from typing import List
+      X = List[Y]
+      Y = List[Z]
+      Z = List[X]
+    """)
+
+    actual_x = ast.Lookup("a.X")
+    expected_x = pytd.Alias(
+        name="a.X",
+        type=pytd.GenericType(
+            base_type=pytd.ClassType("builtins.list"),
+            parameters=(pytd.LateType("Y"),)))
+    self.assertEqual(actual_x, expected_x)
+
+    actual_y = ast.Lookup("a.Y")
+    expected_y = pytd.Alias(
+        name="a.Y",
+        type=pytd.GenericType(
+            base_type=pytd.ClassType("builtins.list"),
+            parameters=(pytd.LateType("Z"),)))
+    self.assertEqual(actual_y, expected_y)
+
+    actual_z = ast.Lookup("a.Z")
+    expected_z = pytd.Alias(
+        name="a.Z",
+        type=pytd.GenericType(
+            base_type=pytd.ClassType("builtins.list"),
+            parameters=(
+                pytd.GenericType(
+                    base_type=pytd.ClassType("builtins.list"),
+                    parameters=(pytd.LateType("Y"),)),)))
+    self.assertEqual(actual_z, expected_z)
+
+
 if __name__ == "__main__":
   unittest.main()
