@@ -6,10 +6,10 @@
 
 import logging
 
-from pytype import abstract
-from pytype import abstract_utils
-from pytype import class_mixin
-from pytype import function
+from pytype.abstract import abstract
+from pytype.abstract import abstract_utils
+from pytype.abstract import class_mixin
+from pytype.abstract import function
 
 log = logging.getLogger(__name__)
 
