@@ -158,7 +158,7 @@ class Converter(utils.ContextWeakrefMixin):
         value = repr(v.value.pyval)
       elif isinstance(v.value.pyval, bool):
         # True and False are stored as pytd constants.
-        value = self.ctx.vm.lookup_builtin(f"builtins.{v.value.pyval}")
+        value = self.ctx.loader.lookup_builtin(f"builtins.{v.value.pyval}")
       else:
         # Ints are stored as their literal values. Note that Literal[None] or a
         # nested literal will never appear here, since we simplified it to None
@@ -218,7 +218,8 @@ class Converter(utils.ContextWeakrefMixin):
     if isinstance(v, (abstract.Empty, typing_overlay.NoReturn)):
       return pytd.NothingType()
     elif isinstance(v, abstract.TypeParameterInstance):
-      if v.module in self._scopes:
+      if (v.module in self._scopes or
+          v.instance is abstract_utils.DUMMY_CONTAINER):
         return self._typeparam_to_def(node, v.param, v.param.name)
       elif v.instance.get_instance_type_parameter(v.full_name).bindings:
         # The type parameter was initialized. Set the view to None, since we
@@ -271,7 +272,10 @@ class Converter(utils.ContextWeakrefMixin):
       return pytd.GenericType(base_type=pytd.NamedType("builtins.type"),
                               parameters=(param,))
     elif isinstance(v, abstract.Module):
-      return pytd.NamedType("builtins.module")
+      if self.ctx.options.gen_stub_imports:
+        return pytd.Alias(v.name, pytd.Module(v.name, module_name=v.full_name))
+      else:
+        return pytd.NamedType("builtins.module")
     elif (self._output_mode >= Converter.OutputMode.LITERAL and
           isinstance(v, abstract.ConcreteValue) and
           isinstance(v.pyval, (int, str, bytes))):
@@ -354,11 +358,17 @@ class Converter(utils.ContextWeakrefMixin):
     Returns:
       A PyTD definition.
     """
-    if isinstance(v, abstract.BoundFunction):
+    if self.ctx.options.gen_stub_imports and isinstance(v, abstract.Module):
+      return pytd.Alias(name, pytd.Module(name, module_name=v.full_name))
+    elif isinstance(v, abstract.BoundFunction):
       d = self.value_to_pytd_def(node, v.underlying, name)
       assert isinstance(d, pytd.Function)
       sigs = tuple(sig.Replace(params=sig.params[1:]) for sig in d.signatures)
       return d.Replace(signatures=sigs)
+    elif isinstance(v, attr_overlay.Attrs):
+      ret = pytd.NamedType("typing.Callable")
+      md = metadata.to_pytd(v.to_metadata())
+      return pytd.Annotated(ret, ("'pytype_metadata'", md))
     elif (isinstance(v, abstract.PyTDFunction) and
           not isinstance(v, typing_overlay.TypeVar)):
       return pytd.Function(
@@ -370,9 +380,7 @@ class Converter(utils.ContextWeakrefMixin):
       return self._function_to_def(node, v, name)
     elif isinstance(v, abstract.SimpleFunction):
       return self._simple_func_to_def(node, v, name)
-    elif (isinstance(v, abstract.ParameterizedClass) or
-          (self.ctx.options.preserve_union_macros and
-           isinstance(v, abstract.Union))):
+    elif isinstance(v, (abstract.ParameterizedClass, abstract.Union)):
       return pytd.Alias(name, v.get_instance_type(node))
     elif isinstance(v, abstract.PyTDClass) and v.module:
       # This happens if a module does e.g. "from x import y as z", i.e., copies
