@@ -5,9 +5,9 @@ is accessed by:
 1. abstract.BuildClass sees a class with enum.Enum as its parent, and calls
 EnumBuilder.make_class.
 2. EnumBuilder.make_class does some validation, then passes along the actual
-creation to vm.make_class. Notably, EnumBuilder passes in EnumInstance to
-vm.make_class, which provides enum-specific behavior.
-3. vm.make_class does its usual, then calls call_metaclass_init on the newly
+creation to ctx.make_class. Notably, EnumBuilder passes in EnumInstance to
+ctx.make_class, which provides enum-specific behavior.
+3. ctx.make_class does its usual, then calls call_metaclass_init on the newly
 created EnumInstance. This bounces back into the overlay, namely EnumMetaInit.
 4. EnumMetaInit does the actual transformation of members into proper enum
 members.
@@ -22,6 +22,9 @@ call_metaclass_init is called, allowing EnumMetaInit to transform the PyTDClass
 into a proper enum.
 """
 
+
+import collections
+import contextlib
 import logging
 
 from pytype import overlay
@@ -87,7 +90,7 @@ class EnumBuilder(abstract.PyTDClass):
           self.ctx.vm.frames, bases[-1], details=msg)
       return node, self.ctx.new_unsolvable(node)
     cls_var = cls_var or self.ctx.vm.loaded_overlays["enum"].members["EnumMeta"]
-    return self.ctx.vm.make_class(
+    return self.ctx.make_class(
         node,
         name_var,
         bases,
@@ -174,7 +177,7 @@ class EnumBuilder(abstract.PyTDClass):
 
     metaclass = self.ctx.vm.loaded_overlays["enum"].members["EnumMeta"]
 
-    return self.ctx.vm.make_class(
+    return self.ctx.make_class(
         node=node,
         name_var=cls_name_var,
         bases=[self.to_variable(node)],
@@ -195,14 +198,24 @@ class EnumInstance(abstract.InterpreterClass):
 
   def __init__(self, name, bases, members, cls, ctx):
     super().__init__(name, bases, members, cls, ctx)
-    # This is set by EnumMetaInit.setup_interpreterclass.
+    # These are set by EnumMetaInit.setup_interpreterclass.
     self.member_type = None
+    self.member_attrs = {}
+    self._instantiating = False
+
+  @contextlib.contextmanager
+  def _is_instantiating(self):
+    old_instantiating = self._instantiating
+    self._instantiating = True
+    try:
+      yield
+    finally:
+      self._instantiating = old_instantiating
 
   def instantiate(self, node, container=None):
     # Instantiate creates a canonical enum member. This intended for when no
     # particular enum member is needed, e.g. during analysis. Real members have
     # these fields set during class creation.
-    # TODO(tsudol): Use the types of other members to set `value`.
     del container
     instance = abstract.Instance(self, self.ctx)
     instance.members["name"] = self.ctx.convert.build_nonatomic_string(node)
@@ -214,6 +227,14 @@ class EnumInstance(abstract.InterpreterClass):
       # But there's no reason not to make sure this function is safe.
       value = self.ctx.new_unsolvable(node)
     instance.members["value"] = value
+    for attr_name, attr_type in self.member_attrs.items():
+      # attr_type might refer back to self, so track whether we are
+      # instantiating to avoid infinite recursion.
+      if self._instantiating:
+        instance.members[attr_name] = self.ctx.new_unsolvable(node)
+      else:
+        with self._is_instantiating():
+          instance.members[attr_name] = attr_type.instantiate(node)
     return instance.to_variable(node)
 
   def is_empty_enum(self):
@@ -247,7 +268,7 @@ class EnumCmpEQ(abstract.SimpleFunction):
     _, argmap = self.match_and_map_args(node, args, alias_map)
     this_var = argmap["self"]
     other_var = argmap["other"]
-    # This is called by vm._call_binop_on_bindings, so both should have
+    # This is called by vm_utils._call_binop_on_bindings, so both should have
     # exactly 1 possibility.
     try:
       this = abstract_utils.get_atomic_value(this_var)
@@ -293,7 +314,7 @@ class EnumMetaInit(abstract.SimpleFunction):
         defaults={},
         annotations={},
         ctx=ctx)
-    self._str_pytd = ctx.vm.lookup_builtin("builtins.str")
+    self._str_pytd = ctx.loader.lookup_builtin("builtins.str")
 
   def _get_class_locals(self, node, cls_name, cls_dict):
     # First, check if get_class_locals works for this class.
@@ -331,10 +352,19 @@ class EnumMetaInit(abstract.SimpleFunction):
     # make all of F's members strings, even if they're assigned a value of
     # a different type.
     # The enum library searches through cls's bases' MRO to find all possible
-    # base types. For simplicity, we just grab the second-to-last base.
+    # base types. For simplicity, we just grab the second-to-last base, but only
+    # if that base type looks valid. (i.e. has its own __new__.)
     if len(bases) > 1:
       base_type_var = bases[-2]
-      return abstract_utils.get_atomic_value(base_type_var, default=None)
+      base_type = abstract_utils.get_atomic_value(base_type_var, default=None)
+      # Pytype's type stubs don't include `__new__` for built-in types like
+      # int, str, complex, etc.
+      if not base_type:
+        return None
+      elif "__new__" in base_type or base_type.full_name.startswith("builtins"):
+        return base_type
+      else:
+        return None
     elif bases and len(bases[0].data) == 1:
       base_type_cls = abstract_utils.get_atomic_value(bases[0])
       if isinstance(base_type_cls, EnumInstance):
@@ -371,6 +401,50 @@ class EnumMetaInit(abstract.SimpleFunction):
       return node, new.to_variable(node)
     return node, None
 
+  def _invalid_name(self, name) -> bool:
+    # "invalid" names mean the value should not be converted to an enum member.
+    # - its name is a dynamic attribute marker.
+    if name in abstract_utils.DYNAMIC_ATTRIBUTE_MARKERS:
+      return True
+    # - its name is a __dunder__ name.
+    # Note that this is not useful in some cases, because __dunder__ names are
+    # not stored where get_class_locals will grab them.
+    if name.startswith("__") and name.endswith("__"):
+      return True
+    return False
+
+  def _is_descriptor(self, node, local) -> bool:
+    # TODO(b/172045608): Revisit this once pytype supports descriptors better.
+    # Descriptors are not converted into enum members. Pytype doesn't currently
+    # support descriptors very well, so this check is a combination of looking
+    # for built-in descriptors (i.e. functions and methods -- NOT all callables)
+    # and anything that has __get__, __set__ or __delete__.
+    def _check(value):
+      if (value.isinstance_Function() or
+          value.isinstance_BoundFunction() or
+          value.isinstance_ClassMethod() or
+          value.isinstance_StaticMethod()):
+        return True
+      for attr_name in ("__get__", "__set__", "__delete__"):
+        _, attr = self.ctx.attribute_handler.get_attribute(
+            node, value, attr_name)
+        if attr is not None:
+          return True
+      return False
+    return any(_check(value) for value in local.orig.data)
+
+  def _not_valid_member(self, node, name, local) -> bool:
+    # Reject a class local if:
+    # - its name is invalid.
+    if self._invalid_name(name):
+      return True
+    # - it has no value. (This will cause an assert to fail later, but we
+    # check it here so we don't crash early on the next check.)
+    if not local.orig:
+      return True
+    # - it's a descriptor.
+    return self._is_descriptor(node, local)
+
   def _is_orig_auto(self, orig):
     try:
       data = abstract_utils.get_atomic_value(orig)
@@ -395,15 +469,27 @@ class EnumMetaInit(abstract.SimpleFunction):
     else:
       return node, self.ctx.convert.build_int(node)
 
-  def _wrap_value(self, node, value, base_type):
-    # Process an enum member's value for use as an argument. Returns a tuple
-    # that can be used for Args.posargs.
-    # If value is not a tuple already, make it one.
-    # Then, if the base type is tuple, wrap it again.
-    arg = abstract_utils.maybe_extract_tuple(value)
-    if base_type.full_name == "builtins.tuple":
-      arg = (self.ctx.convert.build_tuple(node, arg),)
-    return arg
+  def _value_to_starargs(self, node, value_var, base_type):
+    # Process an enum member's value for use as an argument. Returns an
+    # abstract.Tuple that can be used for Args.starargs.
+
+    # If there is more than one option, then we don't know what it is, so just
+    # wrap it in a tuple.
+    if len(value_var.data) > 1:
+      return self.ctx.convert.build_tuple(node, [value_var])
+    # If the value is already a tuple (or a NamedTuple, for example), then use
+    # that value. Otherwise, wrap it in a tuple.
+    value = abstract_utils.get_atomic_value(value_var)
+    if self.ctx.matcher(node).match_from_mro(
+        value.cls, self.ctx.convert.tuple_type):
+      args = value_var
+    else:
+      args = self.ctx.convert.build_tuple(node, [value_var])
+    # However, if the base type of the enum is tuple (and JUST tuple, not a
+    # subclass of tuple) then wrap the args in a tuple again.
+    if base_type and base_type.full_name == "builtins.tuple":
+      args = self.ctx.convert.build_tuple(node, [args])
+    return args
 
   def _mark_dynamic_enum(self, cls):
     # Checks if the enum should be marked as having dynamic attributes.
@@ -430,23 +516,24 @@ class EnumMetaInit(abstract.SimpleFunction):
 
   def _setup_interpreterclass(self, node, cls):
     member_types = []
+    member_attrs = collections.defaultdict(list)
     base_type = self._get_base_type(cls.bases())
+    # Enum members are created by calling __new__ (of either the base type or
+    # the first enum in MRO that defines its own __new__, or else object if
+    # neither of those applies) and then calling __init__ on the member.
+    node, enum_new = self._get_member_new(node, cls, base_type)
     for name, local in self._get_class_locals(node, cls.name, cls.members):
-      if name in abstract_utils.DYNAMIC_ATTRIBUTE_MARKERS:
+      if self._not_valid_member(node, name, local):
         continue
       assert local.orig, ("A local with no assigned value was passed to the "
                           "enum overlay.")
       value = local.orig
       if self._is_orig_auto(value):
         node, value = self._call_generate_next_value(node, cls, name)
-      # Enum members are created by calling __new__ (of either the base type or
-      # the first enum in MRO that defines its own __new__, or else object if
-      # neither of those applies) and then calling __init__ on the member.
-      node, enum_new = self._get_member_new(node, cls, base_type)
       if enum_new:
         new_args = function.Args(
-            posargs=((cls.to_variable(node),) +
-                     abstract_utils.maybe_extract_tuple(value)))
+            posargs=(cls.to_variable(node),),
+            starargs=self._value_to_starargs(node, value, base_type))
         node, member_var = function.call_function(
             self.ctx, node, enum_new, new_args, fallback_to_unsolvable=False)
         # It's possible (but not likely) for member_var to have multiple
@@ -459,7 +546,7 @@ class EnumMetaInit(abstract.SimpleFunction):
               m.cls == member_var.data[0].cls for m in member_var.data):
             member = member_var.data[0]
           else:
-            member_var = self.vm.convert.create_new_unknown(node)
+            member_var = self.ctx.vm.convert.create_new_unknown(node)
             member = abstract_utils.get_atomic_value(member_var)
       else:
         # Build instances directly, because you can't call instantiate() when
@@ -468,24 +555,25 @@ class EnumMetaInit(abstract.SimpleFunction):
         member_var = member.to_variable(node)
       if "_value_" not in member.members:
         if base_type:
-          args = function.Args(posargs=self._wrap_value(node, value, base_type))
+          args = function.Args(
+              posargs=(),
+              starargs=self._value_to_starargs(node, value, base_type))
           node, value = base_type.call(node, base_type.to_binding(node), args)
         member.members["_value_"] = value
       if "__init__" in cls:
         init_args = function.Args(
-            posargs=(member_var,) + abstract_utils.maybe_extract_tuple(value))
+            posargs=(member_var,),
+            starargs=self._value_to_starargs(node, value, base_type))
         node = cls.call_init(node, cls.to_binding(node), init_args)
       member.members["value"] = member.members["_value_"]
       member.members["name"] = self.ctx.convert.build_string(node, name)
+      for attr_name in member.members:
+        if attr_name in ("name", "value"):
+          continue
+        member_attrs[attr_name].extend(member.members[attr_name].data)
       cls.members[name] = member.to_variable(node)
       member_types.extend(value.data)
-    if base_type:
-      member_type = base_type
-    elif member_types:
-      member_type = self.ctx.convert.merge_classes(member_types)
-    else:
-      member_type = self.ctx.convert.unsolvable
-    cls.member_type = member_type
+    # After processing enum members, there's some work to do on the enum itself.
     # If cls has a __new__, save it for later. (See _get_member_new above.)
     # It needs to be marked as a classmethod, or else pytype will try to
     # pass an instance of cls instead of cls when analyzing it.
@@ -498,7 +586,23 @@ class EnumMetaInit(abstract.SimpleFunction):
             node, None, args)
       cls.members["__new_member__"] = saved_new
     self._mark_dynamic_enum(cls)
-    cls.members["__new__"] = self._make_new(node, member_type, cls)
+    if base_type:
+      member_type = base_type
+    elif member_types:
+      member_type = self.ctx.convert.merge_classes(member_types)
+    else:
+      member_type = self.ctx.convert.unsolvable
+    # Only set the lookup-only __new__ on non-empty enums, since using a
+    # non-empty enum for the functional API is a type error.
+    # Note that this has to happen AFTER _mark_dynamic_enum.
+    if member_types:
+      cls.members["__new__"] = self._make_new(node, member_type, cls)
+    cls.member_type = member_type
+
+    member_attrs = {
+        n: self.ctx.convert.merge_classes(ts) for n, ts in member_attrs.items()
+    }
+    cls.member_attrs = member_attrs
     # _generate_next_value_ is used as a static method of the enum, not a class
     # method. We need to rebind it here to make pytype analyze it correctly.
     # However, we skip this if it's already a staticmethod.
@@ -519,7 +623,7 @@ class EnumMetaInit(abstract.SimpleFunction):
     # TODO(tsudol): Ensure only valid enum members are transformed.
     member_types = []
     for pytd_val in cls.pytd_cls.constants:
-      if pytd_val.name in abstract_utils.DYNAMIC_ATTRIBUTE_MARKERS:
+      if self._invalid_name(pytd_val.name):
         continue
       # @property values are instance attributes and must not be converted into
       # enum members. Ideally, these would only be present on the enum members,
@@ -554,11 +658,13 @@ class EnumMetaInit(abstract.SimpleFunction):
     # Because we overwrite __new__, we need to mark dynamic enums here.
     # Of course, this can be moved later once custom __init__ is supported.
     self._mark_dynamic_enum(cls)
-    if not member_types:
-      member_types.append(pytd.AnythingType())
-    member_type = self.ctx.convert.constant_to_value(
-        pytd_utils.JoinTypes(member_types))
-    cls.members["__new__"] = self._make_new(node, member_type, cls)
+    if member_types:
+      member_type = self.ctx.convert.constant_to_value(
+          pytd_utils.JoinTypes(member_types))
+      # Only set the lookup-only __new__ on non-empty enums, since using a
+      # non-empty enum for the functional API is a type error.
+      # Note that this has to happen AFTER _mark_dynamic_enum.
+      cls.members["__new__"] = self._make_new(node, member_type, cls)
     return node
 
   def call(self, node, func, args, alias_map=None):
