@@ -1,10 +1,5 @@
 """Implementation of the types in Python 3's typing.py."""
 
-# We should be able to enable pytype on this file once we switch to the
-# typed_ast-based pyi parser. The current parser can't handle a constant called
-# 'namedtuple':
-# pytype: skip-file
-
 # pylint's detection of this is error-prone:
 # pylint: disable=unpacking-non-sequence
 
@@ -339,7 +334,7 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
     members["_field_defaults"] = field_dict_cls.instantiate(node)
     # _field_types and __annotations__ are both collections.OrderedDicts
     # that map field names (strings) to the types of the fields. Note that
-    # vm.make_class will take care of adding the __annotations__ member.
+    # ctx.make_class will take care of adding the __annotations__ member.
     field_types_cls = abstract.ParameterizedClass(ordered_dict_cls, {
         "K": self.ctx.convert.str_type,
         "V": self.ctx.convert.type_type
@@ -377,6 +372,12 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
         varargs=Param("args"),
         kwargs=Param("kwargs"))
 
+    heterogeneous_tuple_type_params = dict(enumerate(field_types))
+    heterogeneous_tuple_type_params[abstract_utils.T] = field_types_union
+    # Representation of the to-be-created NamedTuple as a typing.Tuple.
+    heterogeneous_tuple_type = abstract.TupleClass(
+        self.ctx.convert.tuple_type, heterogeneous_tuple_type_params, self.ctx)
+
     # _make
     # _make is a classmethod, so it needs to be wrapped by
     # specialibuiltins.ClassMethodInstance.
@@ -422,13 +423,11 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
         kwargs=Param("kwds", field_types_union))
 
     # __getnewargs__
-    getnewargs_tuple_params = dict(
-        tuple(enumerate(field_types)) +
-        ((abstract_utils.T, field_types_union),))
-    getnewargs_tuple = abstract.TupleClass(self.ctx.convert.tuple_type,
-                                           getnewargs_tuple_params, self.ctx)
     members["__getnewargs__"] = overlay_utils.make_method(
-        self.ctx, node, name="__getnewargs__", return_type=getnewargs_tuple)
+        self.ctx,
+        node,
+        name="__getnewargs__",
+        return_type=heterogeneous_tuple_type)
 
     # __getstate__
     members["__getstate__"] = overlay_utils.make_method(
@@ -442,15 +441,20 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
     cls_dict = abstract.Dict(self.ctx)
     cls_dict.update(node, members)
 
+    if self.ctx.options.strict_namedtuple_checks:
+      # Enforces type checking like Tuple[...]
+      superclass_of_new_type = heterogeneous_tuple_type.to_variable(node)
+    else:
+      superclass_of_new_type = self.ctx.convert.tuple_type.to_variable(node)
     if bases:
       final_bases = []
       for base in bases:
         if any(b.full_name == "typing.NamedTuple" for b in base.data):
-          final_bases.append(self.ctx.convert.tuple_type.to_variable(node))
+          final_bases.append(superclass_of_new_type)
         else:
           final_bases.append(base)
     else:
-      final_bases = [self.ctx.convert.tuple_type.to_variable(node)]
+      final_bases = [superclass_of_new_type]
       # This NamedTuple is being created via a function call. We manually
       # construct an annotated_locals entry for it so that __annotations__ is
       # initialized properly for the generated class.
@@ -459,7 +463,7 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
           for field, typ in zip(field_names, field_types)
       }
 
-    node, cls_var = self.ctx.vm.make_class(
+    node, cls_var = self.ctx.make_class(
         node=node,
         name_var=self.ctx.convert.build_string(node, name),
         bases=final_bases,
@@ -656,8 +660,8 @@ class NewType(abstract.PyTDFunction):
         params=[Param(value_arg_name, type_value)])
     members = abstract.Dict(self.ctx)
     members.set_str_item(node, "__init__", constructor)
-    return self.ctx.vm.make_class(node, name_arg, (type_arg,),
-                                  members.to_variable(node), None)
+    return self.ctx.make_class(node, name_arg, (type_arg,),
+                               members.to_variable(node), None)
 
 
 class Overload(abstract.PyTDFunction):
