@@ -24,12 +24,13 @@ def get_parents(
   parents = []
   namedtuple_index = None
   for i, p in enumerate(bases):
-    if _is_parameterized_protocol(p):
-      # From PEP 544: "`Protocol[T, S, ...]` is allowed as a shorthand for
-      # `Protocol, Generic[T, S, ...]`."
-      # https://www.python.org/dev/peps/pep-0544/#generic-protocols
-      parents.append(p.base_type)
-      parents.append(p.Replace(base_type=pytd.NamedType("typing.Generic")))
+    if p.name and pytd_utils.MatchesFullName(p, _PROTOCOL_ALIASES):
+      parents.append(pytd.NamedType("typing.Protocol"))
+      if isinstance(p, pytd.GenericType):
+        # From PEP 544: "`Protocol[T, S, ...]` is allowed as a shorthand for
+        # `Protocol, Generic[T, S, ...]`."
+        # https://www.python.org/dev/peps/pep-0544/#generic-protocols
+        parents.append(p.Replace(base_type=pytd.NamedType("typing.Generic")))
     elif isinstance(p, pytd.NamedType) and p.name == "typing.NamedTuple":
       if namedtuple_index is not None:
         raise ParseError("cannot inherit from bare NamedTuple more than once")
@@ -38,7 +39,7 @@ def get_parents(
     elif isinstance(p, pytd.Type):
       parents.append(p)
     else:
-      msg = "Unexpected class base: " + str(p)
+      msg = f"Unexpected class base: {p}"
       raise ParseError(msg)
 
   return parents, namedtuple_index
@@ -50,7 +51,7 @@ def get_metaclass(keywords: List[ast3.AST], parents: List[pytd_node.Node]):
   for k in keywords:
     keyword, value = k.arg, k.value
     if keyword not in ("metaclass", "total"):
-      raise ParseError("Unexpected classdef kwarg %r" % keyword)
+      raise ParseError(f"Unexpected classdef kwarg {keyword!r}")
     elif keyword == "total" and not any(
         isinstance(parent, pytd.NamedType) and
         pytd_utils.MatchesFullName(parent, _TYPED_DICT_ALIASES)
@@ -74,8 +75,8 @@ def get_decorators(decorators: List[str], type_map: Dict[str, pytd_node.Node]):
   nonclass = {"property", "classmethod", "staticmethod", "overload"}
   unsupported_decorators = set(decorators) & nonclass
   if unsupported_decorators:
-    raise ParseError("Unsupported class decorators: %s" % ", ".join(
-        unsupported_decorators))
+    raise ParseError(
+        f"Unsupported class decorators: {', '.join(unsupported_decorators)}")
 
   # Convert decorators to named types. These are wrapped as aliases because we
   # otherwise do not allow referencing functions as types.
@@ -93,13 +94,4 @@ def check_for_duplicate_defs(methods, constants, aliases) -> None:
                 if count >= 2]
   if duplicates:
     raise ParseError(
-        "Duplicate class-level identifier(s): " + ", ".join(duplicates))
-
-
-#------------------------------------------------------
-# pytd utils
-
-
-def _is_parameterized_protocol(t) -> bool:
-  return (isinstance(t, pytd.GenericType) and
-          pytd_utils.MatchesFullName(t.base_type, _PROTOCOL_ALIASES))
+        f"Duplicate class-level identifier(s): {', '.join(duplicates)}")
