@@ -53,8 +53,8 @@ def make_parser():
       help="Remove mutable parameters.")
   o.add_argument(
       "-V", "--python_version", type=str, action="store",
-      dest="python_version", default="2.7",
-      help=("Python version to target (\"major.minor\", e.g. \"2.7\")"))
+      dest="python_version", default="3",
+      help=("Python version to target (\"major.minor\", e.g. \"3.7\")"))
   o.add_argument(
       "--multiline-args", action="store_true",
       dest="multiline_args", default=False,
@@ -83,7 +83,7 @@ def main():
 
   if opts.optimize:
     parsed = optimize.Optimize(parsed,
-                               builtins.GetBuiltinsPyTD(python_version),
+                               builtins.GetBuiltinsPyTD(),
                                lossy=opts.lossy,
                                use_abcs=opts.use_abcs,
                                max_union=opts.max_union,
