@@ -5,6 +5,7 @@ import copy
 import logging
 import re
 
+from pytype import utils
 from pytype.pytd import base_visitor
 from pytype.pytd import pep484
 from pytype.pytd import pytd
@@ -27,11 +28,15 @@ class PrintVisitor(base_visitor.Visitor):
     self.in_alias = False
     self.in_parameter = False
     self.in_literal = False
-    self._unit_name = None
+    self.in_constant = False
+    self.in_signature = False
+    self.multiline_args = multiline_args
+
+    self._unit = None
     self._local_names = {}
     self._class_members = set()
     self._typing_import_counts = collections.defaultdict(int)
-    self.multiline_args = multiline_args
+    self._module_aliases = {}
 
   def Print(self, node):
     return node.Visit(copy.deepcopy(self))
@@ -107,7 +112,8 @@ class PrintVisitor(base_visitor.Visitor):
   def _NameCollision(self, name):
 
     def name_in(members):
-      return name in members or f"{self._unit_name}.{name}" in members
+      return name in members or (
+          self._unit and f"{self._unit.name}.{name}" in members)
 
     return name_in(self._class_members) or name_in(self._local_names)
 
@@ -130,8 +136,14 @@ class PrintVisitor(base_visitor.Visitor):
     else:
       return self._FromTyping(name)
 
+  def _StripUnitPrefix(self, name):
+    if self._unit:
+      return utils.strip_prefix(name, f"{self._unit.name}.")
+    else:
+      return name
+
   def EnterTypeDeclUnit(self, unit):
-    self._unit_name = unit.name
+    self._unit = unit
     for definitions, label in [(unit.classes, "class"),
                                (unit.functions, "function"),
                                (unit.constants, "constant"),
@@ -139,9 +151,20 @@ class PrintVisitor(base_visitor.Visitor):
                                (unit.aliases, "alias")]:
       for defn in definitions:
         self._local_names[defn.name] = label
+    for alias in unit.aliases:
+      # Modules are represented as NamedTypes in partially resolved asts and
+      # sometimes as LateTypes in asts modified for pickling.
+      if isinstance(alias.type, pytd.Module):
+        module_name = alias.type.module_name
+      elif isinstance(alias.type, (pytd.NamedType, pytd.LateType)):
+        module_name = alias.type.name
+      else:
+        continue
+      name = self._StripUnitPrefix(alias.name)
+      self._module_aliases[module_name] = name
 
   def LeaveTypeDeclUnit(self, _):
-    self._unit_name = None
+    self._unit = None
     self._local_names = {}
 
   def VisitTypeDeclUnit(self, node):
@@ -177,6 +200,12 @@ class PrintVisitor(base_visitor.Visitor):
                           if section_suite)
     return "\n\n".join(sections_as_string)
 
+  def EnterConstant(self, node):
+    self.in_constant = True
+
+  def LeaveConstant(self, node):
+    self.in_constant = False
+
   def VisitConstant(self, node):
     """Convert a class-level or module-level constant to a string."""
     if self.in_literal:
@@ -184,22 +213,21 @@ class PrintVisitor(base_visitor.Visitor):
       assert module == "builtins", module
       assert name in ("True", "False"), name
       return name
-    else:
-      return f"{node.name}: {node.type}"
+    return f"{node.name}: {node.type}"
 
   def EnterAlias(self, _):
     self.old_imports = self.imports.copy()
 
   def VisitAlias(self, node):
     """Convert an import or alias to a string."""
-    if isinstance(self.old_node.type, (pytd.NamedType, pytd.ClassType)):
+    if (isinstance(self.old_node.type,
+                   (pytd.NamedType, pytd.ClassType, pytd.LateType)) and
+        not self.in_constant and not self.in_signature):
       full_name = self.old_node.type.name
       suffix = ""
       module, _, name = full_name.rpartition(".")
       if module:
-        alias_name = self.old_node.name
-        if alias_name.startswith(f"{self._unit_name}."):
-          alias_name = alias_name[len(self._unit_name)+1:]
+        alias_name = self._StripUnitPrefix(self.old_node.name)
         if name not in ("*", alias_name):
           suffix += f" as {alias_name}"
         self.imports = self.old_imports  # undo unnecessary imports change
@@ -301,6 +329,12 @@ class PrintVisitor(base_visitor.Visitor):
     else:
       return self.Print(node.Replace(type=pytd.AnythingType(), optional=False))
 
+  def EnterSignature(self, node):
+    self.in_signature = True
+
+  def LeaveSignature(self, node):
+    self.in_signature = False
+
   def VisitSignature(self, node):
     """Visit a signature, producing a string."""
     if node.return_type == "nothing":
@@ -395,6 +429,26 @@ class PrintVisitor(base_visitor.Visitor):
     """Convert a template to a string."""
     return node.type_param
 
+  def _UseExistingModuleAlias(self, name):
+    prefix, suffix = name.rsplit(".", 1)
+    while prefix:
+      if prefix in self._module_aliases:
+        return f"{self._module_aliases[prefix]}.{suffix}"
+      prefix, _, remainder = prefix.rpartition(".")
+      suffix = f"{remainder}.{suffix}"
+    return None
+
+  def _GuessModule(self, maybe_module):
+    """Guess which part of the given name is the module prefix."""
+    if "." not in maybe_module:
+      return maybe_module
+    prefix, suffix = maybe_module.rsplit(".", 1)
+    # Heuristic: modules are typically lowercase, classes uppercase.
+    if suffix[0].islower():
+      return maybe_module
+    else:
+      return self._GuessModule(prefix)
+
   def VisitNamedType(self, node):
     """Convert a type to a string."""
     prefix, _, suffix = node.name.rpartition(".")
@@ -402,24 +456,23 @@ class PrintVisitor(base_visitor.Visitor):
       node_name = suffix
     elif prefix == "typing":
       node_name = self._FromTyping(suffix)
-    elif (prefix and
-          prefix != self._unit_name and
-          prefix not in self._local_names):
-      if self.class_names and "." in self.class_names[-1]:
-        # We've already fully qualified the class names.
-        class_prefix = self.class_names[-1]
-      else:
-        class_prefix = ".".join(self.class_names)
-      if self._unit_name:
-        class_prefixes = {class_prefix, f"{self._unit_name}.{class_prefix}"}
-      else:
-        class_prefixes = {class_prefix}
-      if prefix not in class_prefixes:
-        # If the prefix doesn't match the class scope, then it's an import.
-        self._RequireImport(prefix)
+    elif "." not in node.name:
       node_name = node.name
     else:
-      node_name = node.name
+      if self._unit:
+        try:
+          pytd.LookupItemRecursive(self._unit, self._StripUnitPrefix(node.name))
+        except KeyError:
+          aliased_name = self._UseExistingModuleAlias(node.name)
+          if aliased_name:
+            node_name = aliased_name
+          else:
+            self._RequireImport(self._GuessModule(prefix))
+            node_name = node.name
+        else:
+          node_name = node.name
+      else:
+        node_name = node.name
     if node_name == "NoneType":
       # PEP 484 allows this special abbreviation.
       return "None"
@@ -449,10 +502,21 @@ class PrintVisitor(base_visitor.Visitor):
     return node.name
 
   def VisitModule(self, node):
-    if node.is_aliased:
-      return f"import {node.module_name} as {node.name}"
-    else:
+    if self.in_constant or self.in_signature:
+      # TODO(slebedev): Use types.ModuleType instead.
+      return "module"
+    elif not node.is_aliased:
       return f"import {node.module_name}"
+    elif "." in node.module_name:
+      # `import x.y as z` and `from x import y as z` are equivalent, but the
+      # latter is a bit prettier.
+      prefix, suffix = node.module_name.rsplit(".", 1)
+      imp = f"from {prefix} import {suffix}"
+      if node.name != suffix:
+        imp += f" as {node.name}"
+      return imp
+    else:
+      return f"import {node.module_name} as {node.name}"
 
   def MaybeCapitalize(self, name):
     """Capitalize a generic type, if necessary."""
