@@ -506,94 +506,3 @@ def MatchesFullName(t, full_name, current_module_name=None, aliases=None):
         expected_module_name,
         parser_constants.EXTERNAL_NAME_PREFIX + expected_module_name}
     return module_name in expected_module_names and name == expected_name
-
-
-def AliasMethod(func, from_constant):
-  """Returns method func with its signature modified as if it has been aliased.
-
-  Args:
-    func: A pytd.Function.
-    from_constant: If True, func will be modified as if it has been aliased from
-      an instance of its defining class, e.g.,
-        class Foo:
-          def func(self): ...
-        const = ...  # type: Foo
-        func = const.func
-      Otherwise, it will be modified as if aliased from the class itself:
-        class Foo:
-          def func(self): ...
-        func = Foo.func
-
-  Returns:
-    A pytd.Function, the aliased method.
-  """
-  # We allow module-level aliases of methods from classes and class instances.
-  # When a static method is aliased, or a normal method is aliased from a class
-  # (not an instance), the entire method signature is copied. Otherwise, the
-  # first parameter ('self' or 'cls') is dropped.
-  new_func = func.Replace(kind=pytd.MethodTypes.METHOD)
-  if func.kind == pytd.MethodTypes.STATICMETHOD or (
-      func.kind == pytd.MethodTypes.METHOD and not from_constant):
-    return new_func
-  return new_func.Replace(signatures=tuple(
-      s.Replace(params=s.params[1:]) for s in new_func.signatures))
-
-
-def LookupItemRecursive(module, name):
-  """Recursively look up name in module."""
-  parts = name.split(".")
-  partial_name = module.name
-  prev_item = None
-  item = module
-
-  def ExtractClass(t):
-    if isinstance(t, pytd.ClassType):
-      return t.cls
-    t = module.Lookup(t.name)  # may raise KeyError
-    if isinstance(t, pytd.Class):
-      return t
-    raise KeyError(t.name)
-
-  for part in parts:
-    prev_item = item
-    # Check the type of item and give up if we encounter a type we don't know
-    # how to handle.
-    if isinstance(item, pytd.Constant):
-      item = ExtractClass(item.type)  # may raise KeyError
-    elif not isinstance(item, (pytd.TypeDeclUnit, pytd.Class)):
-      raise KeyError(name)
-    lookup_name = partial_name + "." + part
-
-    def Lookup(item, *names):
-      for name in names:
-        try:
-          return item.Lookup(name)
-        except KeyError:
-          continue
-      raise KeyError(names[-1])
-
-    # Nested class names are fully qualified while function names are not, so
-    # we try lookup for both naming conventions.
-    try:
-      item = Lookup(item, lookup_name, part)
-    except KeyError:
-      if not isinstance(item, pytd.Class):
-        raise
-      for parent in item.parents:
-        parent_cls = ExtractClass(parent)  # may raise KeyError
-        try:
-          item = Lookup(parent_cls, lookup_name, part)
-        except KeyError:
-          continue  # continue up the MRO
-        else:
-          break  # name found!
-      else:
-        raise  # unresolved
-    if isinstance(item, pytd.Constant):
-      partial_name += "." + item.name.rsplit(".", 1)[-1]
-    else:
-      partial_name = lookup_name
-  if isinstance(item, pytd.Function):
-    return AliasMethod(item, from_constant=isinstance(prev_item, pytd.Constant))
-  else:
-    return item
