@@ -286,7 +286,7 @@ class _ToTypeVisitor(Visitor):
   appropriate allow_constants and allow_functions values.
   """
 
-  def __init__(self, allow_singletons=False):
+  def __init__(self, allow_singletons):
     super().__init__()
     self._in_alias = False
     self._in_literal = 0
@@ -387,7 +387,7 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
         names. If the source contains "import X as Y", module_alias_map should
         contain an entry mapping "Y": "X".
     """
-    super().__init__()
+    super().__init__(allow_singletons=False)
     self._module_map = module_map
     self._module_alias_map = module_alias_map or {}
     self.name = self_name
@@ -471,7 +471,7 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
         self._star_imports.add(module_name)
         item = t  # VisitTypeDeclUnit will remove this unneeded item.
       else:
-        item = pytd_utils.LookupItemRecursive(module, name)
+        item = pytd.LookupItemRecursive(module, name)
     except KeyError as e:
       item = self._ResolveUsingGetattr(module_name, module)
       if item is None:
@@ -559,17 +559,25 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
     Raises:
       KeyError: If there is a name clash.
     """
+    def SameModuleName(a, b):
+      return (
+          isinstance(a.type, pytd.Module) and
+          isinstance(b.type, pytd.Module) and
+          a.type.module_name == b.type.module_name
+      )
+
     name_to_alias = {}
     out = []
     for a in new_aliases:
-      if a.name in name_to_alias:
-        existing = name_to_alias[a.name]
-        if existing != a:
-          raise KeyError("Duplicate top level items: %r, %r" % (
-              existing.type.name, a.type.name))
-      else:
+      if a.name not in name_to_alias:
         name_to_alias[a.name] = a
         out.append(a)
+        continue
+      existing = name_to_alias[a.name]
+      if existing == a or SameModuleName(existing, a):
+        continue
+      raise KeyError("Duplicate top level items: %r, %r" % (
+          existing.type.name, a.type.name))
     return out
 
   def EnterTypeDeclUnit(self, node):
@@ -620,6 +628,11 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
 class LookupLocalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
   """Look up local identifiers. Must be called on a TypeDeclUnit."""
 
+  def __init__(self, allow_singletons=False, toplevel=True):
+    super().__init__(allow_singletons)
+    self._toplevel = toplevel
+    self.local_names = set()
+
   def EnterTypeDeclUnit(self, unit):
     self.unit = unit
 
@@ -627,24 +640,47 @@ class LookupLocalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
     del self.unit
 
   def _LookupItemRecursive(self, name):
-    return pytd_utils.LookupItemRecursive(self.unit, name)
+    return pytd.LookupItemRecursive(self.unit, name)
+
+  def _LookupLocalName(self, node):
+    assert "." not in node.name
+    self.local_names.add(node.name)
+    try:
+      item = self.unit.Lookup(self.unit.name + "." + node.name)
+    except KeyError:
+      # Happens for infer calling load_pytd.resolve_ast() for the final pyi
+      try:
+        item = self.unit.Lookup(node.name)
+      except KeyError as e:
+        if (self.allow_singletons and node.name in pytd.SINGLETON_TYPES):
+          # Let the builtins resolver handle it
+          return node
+        raise SymbolLookupError("Couldn't find %s in %s" % (
+            node.name, self.unit.name)) from e
+    return item
+
+  def _LookupLocalTypes(self, node):
+    visitor = LookupLocalTypes(self.allow_singletons, toplevel=False)
+    visitor.unit = self.unit
+    return node.Visit(visitor), visitor.local_names
 
   def VisitNamedType(self, node):
     """Do lookup on a pytd.NamedType."""
     module_name, dot, name = node.name.rpartition(".")
     if not dot:  # simple reference to a member of the current module
-      try:
-        item = self.unit.Lookup(self.unit.name + "." + node.name)
-      except KeyError:
-        # Happens for infer calling load_pytd.resolve_ast() for the final pyi
-        try:
-          item = self.unit.Lookup(node.name)
-        except KeyError as e:
-          if (self.allow_singletons and node.name in pytd.SINGLETON_TYPES):
-            # Let the builtins resolver handle it
-            return node
-          raise SymbolLookupError("Couldn't find %s in %s" % (
-              node.name, self.unit.name)) from e
+      item = self._LookupLocalName(node)
+      if self._toplevel:
+        # Check if the definition of this name refers back to itself.
+        while isinstance(item, pytd.Alias):
+          new_item, new_item_names = self._LookupLocalTypes(item)
+          if node.name in new_item_names:
+            # We've found a self-reference. This is a recursive type, so delay
+            # resolution by representing it as a LateType.
+            item = pytd.LateType(node.name)
+          elif new_item == item:
+            break
+          else:
+            item = new_item
       try:
         resolved_node = self.to_type(item)
       except NotImplementedError as e:
@@ -663,7 +699,7 @@ class LookupLocalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
     if isinstance(resolved_node, (pytd.Constant, pytd.Function)):
       visitor = LookupLocalTypes()
       visitor.unit = self.unit
-      return resolved_node.Visit(visitor)
+      return self._LookupLocalTypes(resolved_node)[0]
     return resolved_node
 
 
