@@ -18,8 +18,8 @@ class TypeNew(abstract.PyTDFunction):
         bases = list(abstract_utils.get_atomic_python_constant(bases_var))
         if not bases:
           bases = [self.ctx.convert.object_type.to_variable(self.ctx.root_node)]
-        node, variable = self.ctx.vm.make_class(node, name_var, bases,
-                                                class_dict_var, cls)
+        node, variable = self.ctx.make_class(node, name_var, bases,
+                                             class_dict_var, cls)
       except abstract_utils.ConversionError:
         pass
       else:
@@ -331,7 +331,7 @@ class BuiltinClass(abstract.PyTDClass):
 
   def __init__(self, ctx, name, module="builtins"):
     if module == "builtins":
-      pytd_cls = ctx.vm.lookup_builtin("builtins.%s" % name)
+      pytd_cls = ctx.loader.lookup_builtin("builtins.%s" % name)
     else:
       ast = ctx.loader.import_name(module)
       pytd_cls = ast.Lookup("%s.%s" % (module, name))
@@ -694,4 +694,26 @@ class ClassMethod(BuiltinClass):
     arg = args.posargs[0]
     for d in arg.data:
       d.is_classmethod = True
+      d.is_attribute_of_class = True
     return node, ClassMethodInstance(self.ctx, self, arg).to_variable(node)
+
+
+class Dict(BuiltinClass):
+  """Implementation of builtins.dict."""
+
+  def __init__(self, ctx):
+    super().__init__(ctx, "dict")
+
+  def call(self, node, funcb, args):
+    if self.ctx.options.build_dict_literals_from_kwargs:
+      build_literal = not args.has_non_namedargs()
+    else:
+      build_literal = args.is_empty()
+    if build_literal:
+      # special-case a dict constructor with explicit k=v args
+      d = abstract.Dict(self.ctx)
+      for (k, v) in args.namedargs.items():
+        d.set_str_item(node, k, v)
+      return node, d.to_variable(node)
+    else:
+      return super().call(node, funcb, args)
