@@ -4,6 +4,6 @@ from typing import Dict, TypeVar
 _KT = TypeVar('_KT')
 _VT = TypeVar('_VT')
 
-class UserDict(object):
+class UserDict:
     def __init__(self, initialdata: Dict[_KT, _VT] = None):
         pass
