@@ -259,9 +259,9 @@ class CallTracer(vm.VirtualMachine):
     """Set maybe_missing_members to True on these values and their type params.
 
     Args:
-      values: A list of BaseValue objects. On every instance among
-        the values, recursively set maybe_missing_members to True on the
-        instance and its type parameters.
+      values: A list of BaseValue objects. On every instance among the values,
+        recursively set maybe_missing_members to True on the instance and its
+        type parameters.
     """
     values = list(values)
     seen = set()
@@ -288,8 +288,8 @@ class CallTracer(vm.VirtualMachine):
         instantiated to TypeParameterInstance.
       extra_key: Optionally, extra information about the location at which the
         instantion occurs. By default, this method keys on the current opcode
-        and the class, which sometimes isn't enough to disambiguate callers
-        that shouldn't get back the same cached instance.
+        and the class, which sometimes isn't enough to disambiguate callers that
+        shouldn't get back the same cached instance.
 
     Returns:
       A tuple of node and instance variable.
@@ -370,8 +370,7 @@ class CallTracer(vm.VirtualMachine):
       if name in self._CONSTRUCTORS:
         continue  # We already called this method during initialization.
       for v in methodvar.data:
-        if (self.ctx.options.bind_properties and
-            isinstance(v, special_builtins.PropertyInstance)):
+        if isinstance(v, special_builtins.PropertyInstance):
           for m in (v.fget, v.fset, v.fdel):
             if m:
               methods.insert(0, (name, m))
@@ -426,7 +425,7 @@ class CallTracer(vm.VirtualMachine):
       for value in f.bindings:
         if self._should_analyze_as_interpreter_function(value.data):
           node = self.analyze_function(node, value)
-    for func, opcode in self._functions_type_params_check:
+    for func, opcode in self.functions_type_params_check:
       func.signature.check_type_parameter_count(self.simple_stack(opcode))
     return node
 
@@ -497,8 +496,7 @@ class CallTracer(vm.VirtualMachine):
           not all(isinstance(o, abstract.FUNCTION_TYPES) for o in options)):
         if all(
             isinstance(o, (abstract.ParameterizedClass, abstract.TypeParameter,
-                           abstract.Union)) for o in
-            options) and self.ctx.options.preserve_union_macros:  # type alias
+                           abstract.Union)) for o in options):  # type alias
           data.append(
               pytd_utils.JoinTypes(
                   t.to_pytd_def(self.ctx.exitpoint, name) for t in options))
@@ -558,9 +556,11 @@ class CallTracer(vm.VirtualMachine):
     for module_name in ("typing", "typing_extensions"):
       if module_name not in self.loaded_overlays:
         continue
-      module = self.loaded_overlays[module_name].get_module(name)
-      if name in module.members and module.members[name].data == var.data:
-        return True
+      overlay = self.loaded_overlays[module_name]
+      if overlay:
+        module = overlay.get_module(name)
+        if name in module.members and module.members[name].data == var.data:
+          return True
     return False
 
   def pytd_functions_for_call_traces(self):
